@@ -71,9 +71,9 @@
 #include <io.h>
 #include <shlwapi.h>
 #include <direct.h>
-#endif  // defined(_WIN32)
 
 #include "stack_allocation.h"
+#endif  // defined(_WIN32)
 
 #if defined(APPLE_STATIC_LOADER) && !defined(__APPLE__)
 #error "APPLE_STATIC_LOADER can only be defined on Apple platforms!"
@@ -100,10 +100,10 @@
 // Environment Variable information
 #define VK_ICD_FILENAMES_ENV_VAR "VK_ICD_FILENAMES"  // Deprecated in v1.3.207 loader
 #define VK_DRIVER_FILES_ENV_VAR "VK_DRIVER_FILES"
-#define VK_LAYER_PATH_ENV_VAR "VK_LAYER_PATH"
+#define VK_EXPLICIT_LAYER_PATH_ENV_VAR "VK_LAYER_PATH"
 // Support added in v1.3.207 loader
 #define VK_ADDITIONAL_DRIVER_FILES_ENV_VAR "VK_ADD_DRIVER_FILES"
-#define VK_ADDITIONAL_LAYER_PATH_ENV_VAR "VK_ADD_LAYER_PATH"
+#define VK_ADDITIONAL_EXPLICIT_LAYER_PATH_ENV_VAR "VK_ADD_LAYER_PATH"
 // Support added in v1.3.234 loader
 #define VK_LAYERS_ENABLE_ENV_VAR "VK_LOADER_LAYERS_ENABLE"
 #define VK_LAYERS_DISABLE_ENV_VAR "VK_LOADER_LAYERS_DISABLE"
@@ -115,6 +115,9 @@
 #define VK_LOADER_DISABLE_ALL_LAYERS_VAR_3 "**"
 #define VK_LOADER_DISABLE_IMPLICIT_LAYERS_VAR "~implicit~"
 #define VK_LOADER_DISABLE_EXPLICIT_LAYERS_VAR "~explicit~"
+// Support added in v1.3.295 loader
+#define VK_IMPLICIT_LAYER_PATH_ENV_VAR "VK_IMPLICIT_LAYER_PATH"
+#define VK_ADDITIONAL_IMPLICIT_LAYER_PATH_ENV_VAR "VK_ADD_IMPLICIT_LAYER_PATH"
 
 // Override layer information
 #define VK_OVERRIDE_LAYER_NAME "VK_LAYER_LUNARG_override"
@@ -151,8 +154,10 @@
 #define VK_SETTINGS_INFO_REGISTRY_LOC ""
 
 #if defined(__QNX__)
+#ifndef SYSCONFDIR
 #define SYSCONFDIR "/etc"
 #endif
+#endif
 
 // C99:
 #define PRINTF_SIZE_T_SPECIFIER "%zu"
@@ -227,9 +232,14 @@ extern bool loader_disable_dynamic_library_unloading;
 // Returns true if the DIRECTORY_SYMBOL is contained within path
 static inline bool loader_platform_is_path(const char *path) { return strchr(path, DIRECTORY_SYMBOL) != NULL; }
 
-// The once init functionality is not used when building a DLL on Windows. This is because there is no way to clean up the
-// resources allocated by anything allocated by once init. This isn't a problem for static libraries, but it is for dynamic
-// ones. When building a DLL, we use DllMain() instead to allow properly cleaning up resources.
+// The loader has various initialization tasks which it must do before user code can run. This includes initializing synchronization
+// objects, determining the log level, writing the version of the loader to the log, and loading dll's (on windows). On linux, the
+// solution is simply running the initialization code in  __attribute__((constructor)), which MacOS uses when the loader is
+// dynamically linked. When statically linking on MacOS, the setup code instead uses pthread_once to run the logic a single time
+// regardless of which API function the application calls first. On Windows, the equivalent way to run code at dll load time is
+// DllMain which has many limitations placed upon it. Instead, the Windows code follows MacOS and does initialization in the first
+// API call made, using InitOnceExecuteOnce, except for initialization primitives which must be done in DllMain. This is because
+// there is no way to clean up the resources allocated by anything allocated by once init.
 
 #if defined(APPLE_STATIC_LOADER)
 static inline void loader_platform_thread_once_fn(pthread_once_t *ctl, void (*func)(void)) {
@@ -240,6 +250,13 @@ static inline void loader_platform_thread_once_fn(pthread_once_t *ctl, void (*fu
 #define LOADER_PLATFORM_THREAD_ONCE_DECLARATION(var) pthread_once_t var = PTHREAD_ONCE_INIT;
 #define LOADER_PLATFORM_THREAD_ONCE_EXTERN_DEFINITION(var) extern pthread_once_t var;
 #define LOADER_PLATFORM_THREAD_ONCE(ctl, func) loader_platform_thread_once_fn(ctl, func);
+#elif defined(WIN32)
+static inline void loader_platform_thread_win32_once_fn(INIT_ONCE *ctl, PINIT_ONCE_FN func) {
+    InitOnceExecuteOnce(ctl, func, NULL, NULL);
+}
+#define LOADER_PLATFORM_THREAD_ONCE_DECLARATION(var) INIT_ONCE var = INIT_ONCE_STATIC_INIT;
+#define LOADER_PLATFORM_THREAD_ONCE_EXTERN_DEFINITION(var) extern INIT_ONCE var;
+#define LOADER_PLATFORM_THREAD_ONCE(ctl, func) loader_platform_thread_win32_once_fn(ctl, func);
 #else
 #define LOADER_PLATFORM_THREAD_ONCE_DECLARATION(var)
 #define LOADER_PLATFORM_THREAD_ONCE_EXTERN_DEFINITION(var)
@@ -329,7 +346,9 @@ static inline char *loader_platform_executable_path(char *buffer, size_t size) {
 static inline char *loader_platform_executable_path(char *buffer, size_t size) { return NULL; }
 #elif defined(__QNX__)
 
+#ifndef SYSCONFDIR
 #define SYSCONFDIR "/etc"
+#endif
 
 #include <fcntl.h>
 #include <sys/stat.h>
