@@ -59,9 +59,9 @@ enum class GpuType { unspecified, integrated, discrete, external };
 
 struct RegistryEntry {
     RegistryEntry() = default;
-    RegistryEntry(std::string const& name) noexcept : name(name) {}
-    RegistryEntry(std::string const& name, DWORD value) noexcept : name(name), value(value) {}
-    std::string name;
+    RegistryEntry(std::filesystem::path const& name) noexcept : name(name) {}
+    RegistryEntry(std::filesystem::path const& name, DWORD value) noexcept : name(name), value(value) {}
+    std::filesystem::path name;
     DWORD value{};
 };
 
@@ -102,9 +102,9 @@ struct DXGIAdapter {
 };
 
 struct D3DKMT_Adapter {
-    D3DKMT_Adapter& add_driver_manifest_path(fs::path const& src);
-    D3DKMT_Adapter& add_implicit_layer_manifest_path(fs::path const& src);
-    D3DKMT_Adapter& add_explicit_layer_manifest_path(fs::path const& src);
+    D3DKMT_Adapter& add_driver_manifest_path(std::filesystem::path const& src);
+    D3DKMT_Adapter& add_implicit_layer_manifest_path(std::filesystem::path const& src);
+    D3DKMT_Adapter& add_explicit_layer_manifest_path(std::filesystem::path const& src);
 
     UINT hAdapter;
     LUID adapter_luid;
@@ -113,7 +113,7 @@ struct D3DKMT_Adapter {
     std::vector<std::wstring> explicit_layer_paths;
 
    private:
-    D3DKMT_Adapter& add_path(fs::path src, std::vector<std::wstring>& dest);
+    D3DKMT_Adapter& add_path(std::filesystem::path src, std::vector<std::wstring>& dest);
 };
 
 #elif COMMON_UNIX_PLATFORMS
@@ -134,27 +134,33 @@ struct FrameworkEnvironment;  // forward declaration
 // Necessary to have inline definitions as shim is a dll and thus functions
 // defined in the .cpp wont be found by the rest of the application
 struct PlatformShim {
-    PlatformShim() = default;
-    PlatformShim(std::vector<fs::FolderManager>* folders) : folders(folders) {}
+    PlatformShim() { fputs_stderr_log.reserve(65536); }
+    PlatformShim(std::vector<fs::FolderManager>* folders) : folders(folders) { fputs_stderr_log.reserve(65536); }
 
     // Used to get info about which drivers & layers have been added to folders
     std::vector<fs::FolderManager>* folders;
 
+    // Captures the output to stderr from fputs & fputc - aka the output of loader_log()
+    std::string fputs_stderr_log;
+
     // Test Framework interface
     void reset();
 
-    void redirect_all_paths(fs::path const& path);
-    void redirect_category(fs::path const& new_path, ManifestCategory category);
+    void redirect_all_paths(std::filesystem::path const& path);
+    void redirect_category(std::filesystem::path const& new_path, ManifestCategory category);
 
     // fake paths are paths that the loader normally looks in but actually point to locations inside the test framework
-    void set_fake_path(ManifestCategory category, fs::path const& path);
+    void set_fake_path(ManifestCategory category, std::filesystem::path const& path);
 
     // known paths are real paths but since the test framework guarantee's the order files are found in, files in these paths
     // need to be ordered correctly
-    void add_known_path(fs::path const& path);
+    void add_known_path(std::filesystem::path const& path);
 
-    void add_manifest(ManifestCategory category, fs::path const& path);
-    void add_unsecured_manifest(ManifestCategory category, fs::path const& path);
+    void add_manifest(ManifestCategory category, std::filesystem::path const& path);
+    void add_unsecured_manifest(ManifestCategory category, std::filesystem::path const& path);
+
+    void clear_logs() { fputs_stderr_log.clear(); }
+    bool find_in_log(std::string const& search_text) const { return fputs_stderr_log.find(search_text) != std::string::npos; }
 
 // platform specific shim interface
 #if defined(WIN32)
@@ -164,20 +170,20 @@ struct PlatformShim {
 
     void add_dxgi_adapter(GpuType gpu_preference, DXGI_ADAPTER_DESC1 desc1);
     void add_d3dkmt_adapter(D3DKMT_Adapter const& adapter);
-    void set_app_package_path(fs::path const& path);
+    void set_app_package_path(std::filesystem::path const& path);
 
     std::unordered_map<uint32_t, DXGIAdapter> dxgi_adapters;
 
     std::vector<D3DKMT_Adapter> d3dkmt_adapters;
 
     // TODO:
-    void add_CM_Device_ID(std::wstring const& id, fs::path const& icd_path, fs::path const& layer_path);
+    void add_CM_Device_ID(std::wstring const& id, std::filesystem::path const& icd_path, std::filesystem::path const& layer_path);
     std::wstring CM_device_ID_list = {L'\0'};
     std::vector<RegistryEntry> CM_device_ID_registry_keys;
 
     uint32_t random_base_path = 0;
 
-    std::vector<fs::path> icd_paths;
+    std::vector<std::filesystem::path> icd_paths;
 
     std::vector<RegistryEntry> hkey_current_user_explicit_layers;
     std::vector<RegistryEntry> hkey_current_user_implicit_layers;
@@ -194,22 +200,22 @@ struct PlatformShim {
     std::vector<HKeyHandle> created_keys;
 
 #elif COMMON_UNIX_PLATFORMS
-    bool is_fake_path(fs::path const& path);
-    fs::path const& get_real_path_from_fake_path(fs::path const& path);
+    bool is_fake_path(std::filesystem::path const& path);
+    std::filesystem::path const& get_real_path_from_fake_path(std::filesystem::path const& path);
 
-    void redirect_path(fs::path const& path, fs::path const& new_path);
-    void remove_redirect(fs::path const& path);
+    void redirect_path(std::filesystem::path const& path, std::filesystem::path const& new_path);
+    void remove_redirect(std::filesystem::path const& path);
 
-    bool is_known_path(fs::path const& path);
-    void remove_known_path(fs::path const& path);
+    bool is_known_path(std::filesystem::path const& path);
+    void remove_known_path(std::filesystem::path const& path);
 
-    void redirect_dlopen_name(fs::path const& filename, fs::path const& actual_path);
-    bool is_dlopen_redirect_name(fs::path const& filename);
+    void redirect_dlopen_name(std::filesystem::path const& filename, std::filesystem::path const& actual_path);
+    bool is_dlopen_redirect_name(std::filesystem::path const& filename);
 
-    fs::path query_default_redirect_path(ManifestCategory category);
+    std::filesystem::path query_default_redirect_path(ManifestCategory category);
 
-    std::unordered_map<std::string, fs::path> redirection_map;
-    std::unordered_map<std::string, fs::path> dlopen_redirection_map;
+    std::unordered_map<std::string, std::filesystem::path> redirection_map;
+    std::unordered_map<std::string, std::filesystem::path> dlopen_redirection_map;
     std::unordered_set<std::string> known_path_set;
 
     void set_elevated_privilege(bool elev) { use_fake_elevation = elev; }
@@ -227,7 +233,8 @@ struct PlatformShim {
 std::vector<std::string> parse_env_var_list(std::string const& var);
 std::string category_path_name(ManifestCategory category);
 
-std::vector<std::string> get_folder_contents(std::vector<fs::FolderManager>* folders, std::string folder_name) noexcept;
+std::vector<std::filesystem::path> get_folder_contents(std::vector<fs::FolderManager>* folders,
+                                                       std::filesystem::path folder_name) noexcept;
 
 extern "C" {
 // dynamically link on windows and macos
@@ -235,7 +242,7 @@ extern "C" {
 using PFN_get_platform_shim = PlatformShim* (*)(std::vector<fs::FolderManager>* folders);
 #define GET_PLATFORM_SHIM_STR "get_platform_shim"
 
-#elif defined(__linux__) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__GNU__)
+#elif defined(__linux__) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__GNU__) || defined(__QNX__)
 // statically link on linux
 PlatformShim* get_platform_shim(std::vector<fs::FolderManager>* folders);
 #endif
