@@ -850,3 +850,127 @@ TEST(WsiTests, SwapchainFunctional) {
     }
     env.vulkan_functions.vkDestroySurfaceKHR(inst.inst, surface, nullptr);
 }
+
+TEST(WsiTests, EXTSurfaceMaintenance1) {
+    FrameworkEnvironment env{};
+
+    std::vector<VkPresentModeKHR> present_modes{VK_PRESENT_MODE_FIFO_KHR, VK_PRESENT_MODE_IMMEDIATE_KHR,
+                                                VK_PRESENT_MODE_MAILBOX_KHR, VK_PRESENT_MODE_FIFO_RELAXED_KHR};
+    VkSurfaceCapabilitiesKHR surface_caps{};
+    surface_caps.maxImageExtent = VkExtent2D{300, 300};
+    surface_caps.minImageExtent = VkExtent2D{100, 100};
+    env.add_icd(TestICDDetails(TEST_ICD_PATH_VERSION_2))
+        .setup_WSI()
+        .add_instance_extension(VK_KHR_GET_SURFACE_CAPABILITIES_2_EXTENSION_NAME)
+        .add_physical_device(PhysicalDevice{}
+                                 .add_extension("VK_KHR_swapchain")
+                                 .set_deviceName("no")
+                                 .set_surface_capabilities(surface_caps)
+                                 .add_surface_present_modes(present_modes)
+                                 .finish());
+    VkSurfacePresentScalingCapabilitiesEXT scaling_capabilities{};
+    scaling_capabilities.supportedPresentScaling = VK_PRESENT_SCALING_ONE_TO_ONE_BIT_EXT;
+    scaling_capabilities.supportedPresentGravityX = VK_PRESENT_SCALING_ASPECT_RATIO_STRETCH_BIT_EXT;
+    scaling_capabilities.supportedPresentGravityY = VK_PRESENT_SCALING_STRETCH_BIT_EXT;
+    scaling_capabilities.minScaledImageExtent = {60, 60};
+    scaling_capabilities.maxScaledImageExtent = {1000, 1000};
+    auto& icd2 = env.add_icd(TestICDDetails(TEST_ICD_PATH_VERSION_2))
+                     .setup_WSI()
+                     .add_instance_extension(VK_EXT_SURFACE_MAINTENANCE_1_EXTENSION_NAME)
+                     .add_instance_extension(VK_KHR_GET_SURFACE_CAPABILITIES_2_EXTENSION_NAME)
+                     .add_physical_device(PhysicalDevice{}
+                                              .add_extension("VK_KHR_swapchain")
+                                              .set_deviceName("yes")
+                                              .set_surface_capabilities(surface_caps)
+                                              .add_surface_present_modes(present_modes)
+                                              .set_surface_present_scaling_capabilities(scaling_capabilities)
+                                              .finish());
+    std::vector<std::vector<VkPresentModeKHR>> compatible_present_modes{
+        {VK_PRESENT_MODE_FIFO_KHR, VK_PRESENT_MODE_FIFO_RELAXED_KHR},
+        {VK_PRESENT_MODE_IMMEDIATE_KHR, VK_PRESENT_MODE_MAILBOX_KHR},
+        {VK_PRESENT_MODE_MAILBOX_KHR, VK_PRESENT_MODE_IMMEDIATE_KHR},
+        {VK_PRESENT_MODE_FIFO_RELAXED_KHR, VK_PRESENT_MODE_FIFO_KHR},
+    };
+    icd2.physical_devices[0].surface_present_mode_compatibility = compatible_present_modes;
+    env.add_icd(TestICDDetails(TEST_ICD_PATH_VERSION_2))
+        .setup_WSI()
+        .add_physical_device(PhysicalDevice{}
+                                 .add_extension("VK_KHR_swapchain")
+                                 .set_deviceName("no")
+                                 .set_surface_capabilities(surface_caps)
+                                 .add_surface_present_modes(present_modes)
+                                 .finish());
+
+    InstWrapper inst{env.vulkan_functions};
+    inst.create_info.setup_WSI();
+    inst.create_info.add_extension(VK_EXT_SURFACE_MAINTENANCE_1_EXTENSION_NAME);
+    inst.create_info.add_extension(VK_KHR_GET_SURFACE_CAPABILITIES_2_EXTENSION_NAME);
+    inst.CheckCreate();
+
+    VkSurfaceKHR surface{};
+    ASSERT_EQ(VK_SUCCESS, create_surface(inst, surface));
+    WrappedHandle<VkSurfaceKHR, VkInstance, PFN_vkDestroySurfaceKHR> wrapped_surface{surface, inst.inst,
+                                                                                     env.vulkan_functions.vkDestroySurfaceKHR};
+    auto physical_devices = inst.GetPhysDevs(3);
+
+    for (auto physical_device : physical_devices) {
+        VkPhysicalDeviceProperties phys_dev_props{};
+        inst->vkGetPhysicalDeviceProperties(physical_device, &phys_dev_props);
+        bool driver_support_surface_maintenance1 = string_eq(phys_dev_props.deviceName, "yes");
+
+        uint32_t present_mode_count = 4;
+        std::vector<VkPresentModeKHR> queried_present_modes{present_mode_count};
+        inst->vkGetPhysicalDeviceSurfacePresentModesKHR(physical_device, surface, &present_mode_count,
+                                                        queried_present_modes.data());
+
+        for (uint32_t i = 0; i < present_mode_count; i++) {
+            VkSurfacePresentModeEXT present_mode{};
+            present_mode.sType = VK_STRUCTURE_TYPE_SURFACE_PRESENT_MODE_EXT;
+            present_mode.presentMode = queried_present_modes[i];
+
+            VkPhysicalDeviceSurfaceInfo2KHR surface_info{};
+            surface_info.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SURFACE_INFO_2_KHR;
+            surface_info.surface = surface;
+            surface_info.pNext = &present_mode;
+
+            VkSurfacePresentModeCompatibilityEXT SurfacePresentModeCompatibilityEXT{};
+            SurfacePresentModeCompatibilityEXT.sType = VK_STRUCTURE_TYPE_SURFACE_PRESENT_MODE_COMPATIBILITY_EXT;
+
+            VkSurfacePresentScalingCapabilitiesEXT SurfacePresentScalingCapabilitiesEXT{};
+            SurfacePresentScalingCapabilitiesEXT.sType = VK_STRUCTURE_TYPE_SURFACE_PRESENT_SCALING_CAPABILITIES_EXT;
+            SurfacePresentScalingCapabilitiesEXT.pNext = &SurfacePresentModeCompatibilityEXT;
+
+            VkSurfaceCapabilities2KHR surface_caps2{};
+            surface_caps2.sType = VK_STRUCTURE_TYPE_SURFACE_CAPABILITIES_2_KHR;
+            surface_caps2.pNext = &SurfacePresentScalingCapabilitiesEXT;
+
+            ASSERT_EQ(VK_SUCCESS, env.vulkan_functions.vkGetPhysicalDeviceSurfaceCapabilities2KHR(physical_device, &surface_info,
+                                                                                                  &surface_caps2));
+            if (driver_support_surface_maintenance1) {
+                ASSERT_EQ(SurfacePresentModeCompatibilityEXT.presentModeCount, 2U);
+            } else {
+                ASSERT_EQ(SurfacePresentModeCompatibilityEXT.presentModeCount, 1U);
+            }
+            std::vector<VkPresentModeKHR> queried_compatible_present_modes{SurfacePresentModeCompatibilityEXT.presentModeCount};
+            SurfacePresentModeCompatibilityEXT.pPresentModes = queried_compatible_present_modes.data();
+
+            ASSERT_EQ(VK_SUCCESS, env.vulkan_functions.vkGetPhysicalDeviceSurfaceCapabilities2KHR(physical_device, &surface_info,
+                                                                                                  &surface_caps2));
+
+            if (driver_support_surface_maintenance1) {
+                ASSERT_EQ(compatible_present_modes[i], queried_compatible_present_modes);
+                ASSERT_EQ(scaling_capabilities, SurfacePresentScalingCapabilitiesEXT);
+            } else {
+                // Make sure the emulation returned the values we expect - 1 compatible present mode which is the mode we are
+                // querying, Scaling capabilities is 0 (aka none) and the extent is just the surface caps extent
+                ASSERT_EQ(SurfacePresentModeCompatibilityEXT.presentModeCount, 1U);
+                ASSERT_EQ(SurfacePresentModeCompatibilityEXT.pPresentModes[0], queried_present_modes[i]);
+                ASSERT_EQ(SurfacePresentScalingCapabilitiesEXT.supportedPresentScaling, 0U);
+                ASSERT_EQ(SurfacePresentScalingCapabilitiesEXT.supportedPresentGravityX, 0u);
+                ASSERT_EQ(SurfacePresentScalingCapabilitiesEXT.supportedPresentGravityY, 0U);
+                ASSERT_EQ(SurfacePresentScalingCapabilitiesEXT.minScaledImageExtent, surface_caps.minImageExtent);
+                ASSERT_EQ(SurfacePresentScalingCapabilitiesEXT.maxScaledImageExtent, surface_caps.maxImageExtent);
+            }
+        }
+    }
+}
